#!/usr/local/bin/perl

# Michael Ashley / 26-Sep-1998 / UNSW

# Reads a file of the format:
#
#    ExpT   Zero      Signal    Expt    Zero     Signal
#    5.00 2014.924   3231.472    1.00 2015.356    646.346
#    5.00 2015.492   3231.579    2.00 2015.516   1292.192
#    5.00 2015.612   3231.411    3.00 2015.612   1937.202
#
# and produces a file of the format:
#
#     Actual    Desired
#    646.346    645.093
#   1292.192   1290.174
#   1937.202   1935.180
#
# Where the "desired" signal is calculated from the observed signal
# in the reference observation (at T=5secs in the above example).
# A linear interpolation is done to estimate the T=5 intensity at
# the midpoint of the observation.
#
# Here is the old linearity correction (from 1995):
#
#   imexpr ("a < 15700 ? a*(1-6e-7*a) : a*(0.9424+3.07e-6*a)", outfile, j1)
#
# Here is the new one (26 Sep 1998):
#
# "a<12000 ? a*(1-6.8e-7*a) : 806+a*(0.8586+a*(5.90e-6-a*3.12e-11))"
#
# These are for "gain lo speed lo".
#

$PrevRef = 0;
$Scale = 1.001879; # Chosen to give a 1.00 coefficient in the formula
                   # for low fluxes.
while (<>) {
    if (!/^\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)$/) {
	die "bad format $_";
    }
    $NextRef = $3;
    if ($PrevRef != 0) {
	$AverageRef = ($NextRef + $PrevRef)/2.0;
	printf ("%10.3f %10.3f\n", $PrevSignal, 
           $PrevTime * $AverageRef / ($1 * $Scale));
    }
    $PrevTime   = $4;
    $PrevSignal = $6;
    $PrevRef    = $NextRef;
}
