#include <stdio.h>
/* #include <math.h> */
#include <sys/types.h>

#include "saocat.h"
;

#define max(A,B) ((A) > (B) ? (A) : (B))
#define min(A,B) ((A) < (B) ? (A) : (B))

/* Michael Ashley / UNSW / 27-Apr-1999 */

void usage (char *program) {
  printf ("Usage: %s ra_centre dec_centre delta_ra delta_dec\n",
    program);
  printf ("        all numbers in radians.\n");
  exit (1);
}

int main (int argc, char *argv[]) {

  double    Ra, Dec, DeltaRa, DeltaDec;
  const double pi = 3.14159265359;
  const unsigned long int Two32m1 = 4294967295UL;
  const long int Two31m1 = 2147483647L;

  double    ramult, decmult, x, y;
  double    t;
  unsigned  long Ra1,  Ra2;
  long      Dec1, Dec2;
  long      MaxSao, i;

  ramult  = (Two32m1+1.0)/(2*pi);
  decmult = (Two31m1+1.0)/pi;

  if (argc != 5) {
    usage (argv[0]);
  }

  sscanf (argv[1], "%lf", &Ra);
  sscanf (argv[2], "%lf", &Dec);
  sscanf (argv[3], "%lf", &DeltaRa);
  sscanf (argv[4], "%lf", &DeltaDec);

  t = (Ra - DeltaRa) * ramult + 0.5;
  Ra1 = max(min(t,Two32m1),0);

  t = (Dec - DeltaDec) * decmult + 0.5;
  Dec1 = max(min(t,Two31m1),-Two31m1-1);

  t = (Ra + DeltaRa) * ramult + 0.5;
  Ra2 = max(min(t,Two32m1),0);

  t = (Dec + DeltaDec) * decmult + 0.5;
  Dec2 = max(min(t,Two31m1),-Two31m1 -1);

  MaxSao = (sizeof (SaoCatalog)) / 8;

  /*    printf ("%lf %lf %ld %ld %ld %ld %ld\n", Ra, Dec, Ra1, Ra2, Dec1, Dec2, MaxSao); */


  for (i = 0; i < MaxSao; i++) {
    if (SaoCatalog[i].ra  > Ra1  && SaoCatalog[i].ra  < Ra2 &&
        SaoCatalog[i].dec > Dec1 && SaoCatalog[i].dec < Dec2) {
      x = SaoCatalog[i].ra / ramult;
      y = SaoCatalog[i].dec / decmult;
      printf ("%10.8lf %11.8lf %6d\n", x, y, i+1);
    }
  }
}

