include <imhdr.h>
include <math.h>
include <mach.h>
include <fset.h>
include <gset.h>

.help cubestat May98 unsw 
.ih
NAME
cubestat -- 
.ih
USAGE
xc cubestat.x
task cubestat=/usr/users/astro/mcba/apt/cubestat.e
.ih
PARAMETERS
.ls cube
The input cube consists of a 1-D stack of 2-D 
.ih
AUTHOR
Author: Michael Ashley / UNSW / 06-May-1998 / mcba@newt.phys.unsw.edu.au
.endhelp

task cubestat = t_cubestat

#procedure t_cubestat (cube, im_min, im_max)
procedure t_cubestat ()

char cube[SZ_FNAME]             # input data
char im_min[SZ_FNAME] 
char im_max[SZ_FNAME]

pointer imc, iMIN, iMAX, bufc, bufMIN, bufMAX

pointer immap(), imgs3r(), imps2r(), imgs2r()
int     imaccess()
real    clgetr()
bool    clgetb(), streq(), strne()

int     nx, ny, nz, i, j

begin

# Obtain the input cube filename, and the output filename.

  call clgstr ("cube",      cube,    SZ_FNAME)
  call clgstr ("im_min",    im_min,  SZ_FNAME)
  call clgstr ("im_max",    im_max,  SZ_FNAME)

# Open the input cube, and find its dimensions.

  imc = immap (cube, READ_ONLY, 0)
  nx  = IM_LEN(imc, 1)
  ny  = IM_LEN(imc, 2)
  nz  = IM_LEN(imc, 3)

# Now we delete any existing images that conflict with the ones that
# we want to create. This isn't in the spirit of IRAF, but I get fed
# up with deleting images manually...

  if (imaccess (im_min,   READ_ONLY) == YES) {
    call imdelete (im_min)
  }
  if (imaccess (im_max,  READ_ONLY) == YES) {
    call imdelete (im_max)
  }

# Open the various output files.

  iMIN = immap (im_min, NEW_IMAGE, 28800)
  IM_PIXTYPE(iMIN) = TY_REAL
  IM_NDIM(iMIN)    = 2
  IM_LEN(iMIN,1)   = nx
  IM_LEN(iMIN,2)   = ny

  iMAX = immap (im_max, NEW_IMAGE, 28800)
  IM_PIXTYPE(iMAX) = TY_REAL
  IM_NDIM(iMAX)    = 2
  IM_LEN(iMAX,1)   = nx
  IM_LEN(iMAX,2)   = ny

# Read the entire cube into memory, and prepare memory areas for the output
# images.

  bufc = imgs3r (imc, 1, nx, 1, ny, 1, nz)
  bufMIN = imps2r (iMIN, 1, nx, 1, ny)
  bufMAX = imps2r (iMAX, 1, nx, 1, ny)

  do j = 1, ny {
    do i = 1, nx {
      call process (Memr[bufc], Memr[bufMIN], Memr[bufMAX], 
                    nx, ny, nz, i, j)
    }
  }

# Close all the images.

  call imunmap (imc)
  call imunmap (iMIN)
  call imunmap (iMAX)

end

procedure process (imc, imMIN, imMAX, nx, ny, nz, 
                   cur_x, cur_y)

# Author: Michael Ashley / UNSW / 06-May-1998 / mcba@newt.phys.unsw.edu.au

real imc[nx, ny, nz]            # input cube
real imMIN[nx, ny] 
real imMAX[nx, ny] 
int  nx, ny, nz                 # dimensions of input cube
int  cur_x, cur_y               # (x,y) coordinates currently being worked on

int  i
real dmin, dmax

begin

  dmin = imc[cur_x, cur_y, 1]
  dmax = dmin

  do i = 2, nz {
    dmin = min (dmin, imc[cur_x, cur_y, i])
    dmax = max (dmax, imc[cur_x, cur_y, i])
  }
  
  imMIN[cur_x, cur_y] = dmin
  imMAX[cur_x, cur_y] = dmax

end
