procedure find_stars (image, output, fwhmpsf, sigback, threshold, verbose)

#
# Michael Ashley / UNSW / 29-Apr-1998
#
# A wrapper for daofind, suitable for use with the APT.
#
# Type the following command into the IRAF cl to use this program:
#
# task find_stars =/usr/users/astro/mcba/apt/find_stars.cl
#

string image=""          {prompt="Image in which to search for stars"}
string output=""         {prompt="Output file to contain the star coords"}
real   fwhmpsf=1.6       {prompt="FWHM of the PSF in pixels"}
real   sigback=10.0      {prompt="Standard dev of background in counts"}
real   threshold=40.0    {prompt="Threshold in sigma for feature detection"}
bool   verbose=no        {prompt="Verbose output?"}

struct *IMGLIST

begin

# Copies of the parameters.

   string Iimage, Ioutput
   real   Ifwhmpsf, Isigback, Ithreshold
   bool   Iverbose

# Local variables.

   string imgfile, img
   int    nimg

# Assign positional parameters to local variables.

   Iimage     = image
   Ioutput    = output
   Ifwhmpsf   = fwhmpsf
   Isigback   = sigback
   Ithreshold = threshold
   Iverbose   = verbose

# Form temporary filenames.

   imgfile =  mktemp ("/tmp/mcba")

# Restore defaults for all the IRAF routines we use.

   unlearn daofind
   unlearn datapars
   unlearn findpars

# Set all the relevant parameters. The important ones that may need changing
# are indicated with two ##

   datapars.scale      =         1. # Image scale in units per pixel
   datapars.fwhmpsf    =   Ifwhmpsf ## FWHM of the PSF in scale units
   datapars.emission   =        yes # Features are positive ?
   datapars.sigma      =   Isigback ## Standard dev of background in counts
   datapars.datamin    =         0. # Minimum good data value
   datapars.datamax    =     65000. # Maximum good data value
   datapars.noise      =  "poisson" # Noise model
   datapars.ccdread    =  "CCDREAD" # CCD readout noise image header keyword
   datapars.gain       =  "CCDGAIN" # CCD gain image header keyword
   datapars.readnoise  =         0. # CCD readout noise in electrons
   datapars.epadu      =         1. # Gain in electrons per count
   datapars.exposure   =  "EXPTIME" # Exposure time image header keyword
   datapars.airmass    =  "AIRMASS" # Airmass image header keyword
   datapars.filter     =   "SUBSET" # Filter image header keyword
   datapars.obstime    = "TIME-BEG" # Time of observation image header keyword

   findpars.threshold  = Ithreshold ## Threshold in sigma for feature detection
   findpars.nsigma     = 1.5  # Width of convolution kernel in sigma
   findpars.ratio      = 1.0  # Ratio of minor to major axis of Gaussian kernel
   findpars.theta      = 0.0  # Position angle of major axis of Gaussian kernel
   findpars.sharplo    = 0.1  # Lower bound on sharpness for feature detection
   findpars.sharphi    = 1.0  # Upper bound on sharpness for  feature detection
   findpars.roundlo    = -1.0 # Lower bound on roundness for feature detection
   findpars.roundhi    = 1.0  # Upper bound on roundness for feature detection
   findpars.mkdetect   = yes  # Mark detections on the image display ?

   daofind.datapar     = ""
   daofind.findpar     = ""
   daofind.boundary    = "nearest"
   daofind.constant    = 0.0
   daofind.interact    = no
   daofind.verify      = no
   daofind.verbose     = Iverbose

# Check existence of input @file.

   if ( (stridx("@",Iimage) == 1) &&
        (!access( substr(Iimage,2,strlen(Iimage))) ) ) {
      print ("FIND_STARS: Input image @file ",Iimage," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }

# Expand input template into file lists.

   sections (Iimage, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("FIND_STARS: No images in input - processing aborted.")
      beep ; goto cleanup
   }
   IMGLIST = imgfile

# Get the first image name.

   if (fscan (IMGLIST,img) == EOF) {
      print ("FIND_STARS: No images in input - processing aborted.")
      beep ; goto cleanup
   }
   
# Delete the output file.

   delete (Ioutput, ver-, >& "dev$null")

# Find the stars!

   daofind (image=img, output=Ioutput)

end
