#
# FIND_VAR -- 
#		
#               
#
#
# e.g., find_var (@images, ver+)
#
# History: 24-APR-98  JMR and MCBA.
# 
# task find_var=/data1/jmr/980403/BC_vir/tables/find_var.cl
# task find_var=/usr/users/astro/mcba/apt/find_var.cl
# task find_var=scripts$find_var.cl

procedure find_var (images, offsets, xy_file, calc_offsets, verbose)

string images           {prompt="List of input images"}
string offsets          {prompt="File containing xy offsets of each image"}
string xy_file          {prompt="File containing xy coordinates of the stars"}
bool   calc_offsets     {prompt="Should we refine the xy offsets?"}
bool   verbose          {no,  prompt="Verbose output?"}

struct *OFFSETS
struct *XY_FILE
struct *IMGLIST


begin

# Copies of the parameters.

   string Iimages, Ioffsets, Ixy_file
   bool   Icalc_offsets, Iverbose

# Local variables.

   string imgfile, center_file, otime
   int    nimg, i, num_images, nx, ny, nrows, extra_rows, npixels
   struct img
   real   x_offset, y_offset, x_zero, y_zero
   bool   first
   
# Assign positional parameters to local variables.

   Iimages       = images
   Ioffsets      = offsets
   Ixy_file      = xy_file
   Icalc_offsets = calc_offsets
   Iverbose      = verbose

# Restore defaults for all the IRAF routines we use.

   unlearn sections
   unlearn tproject
   unlearn tmerge
   unlearn phot
   unlearn tcreate
   unlearn imcentroid

   first = yes

# Form temporary filenames.

   imgfile     =  mktemp("/tmp/ccd")
   center_file =  mktemp("/tmp/cf")

# Check existence of input @file.

   if ( (stridx("@",Iimages) == 1) &&
        (! access( substr(Iimages,2,strlen(Iimages))) ) ) {
      print ("FIND_VAR: Input image @file ",Iimages," does not exist",
         " - processing aborted.")
      beep ; goto cleanup
   }


# Expand input template into file lists.

   sections (Iimages, option="fullname", > imgfile)
   nimg = sections.nimages
   if (nimg == 0) {
      print ("FIND_VAR: No images in input list - processing aborted.")
      beep ; goto cleanup
   }


# Check existence of offset and xy files.

   if (!access(Ioffsets)) {
      print ("FIND_VAR: Can't access offsets file.")
      beep ; goto cleanup
   }

   if (!access(Ixy_file)) {
      print ("FIND_VAR: Can't access xy_file coordinate file.")
      beep ; goto cleanup
   }


#  File handles

   IMGLIST = imgfile

#  Remove all photometry output files in the current directory.

   delete ("*.mag.1", ver-, >& "dev$null")

   num_images = 0
   i = 0 
	
# Scan through input images.

   while (fscan (IMGLIST,img) != EOF) {
      if ( !access (img) && !access (img//".fits") && !access (img//".imh") ) {
         print ("FIND_VAR: Image ",img," does not exist")
         beep ; goto cleanup
      } else {
         if (stridx('.',img) != 0) {
            img = substr(img,1,stridx('.',img)-1)
         }

   	 if (first) {

           if (Icalc_offsets) {
             printf ("\nRefining images offsets ...")
             delete ("junk.c", ver-, >& "dev$null")
#            delete ("junk.cen")
             imcentroid (input=Iimages, reference=img, coords=Ixy_file,
                         boxsize=5,
                         shifts=Ioffsets, verbose-, >& "junk.c")
             goto cleanup
           }
           OFFSETS = Ioffsets
           XY_FILE = Ixy_file
         }

         printf ("\nProcessing %s ...", img)
         num_images += 1  
	 i += 1
    
#  Defining parameters so that the offset, relative to the first image is 
#    calculated.

      if (fscan(OFFSETS, x_offset, y_offset) != EOF) {
         if (first) {
           x_zero = x_offset
           y_zero = y_offset
           first = no
         }
         x_offset -= x_zero
         y_offset -= y_zero


#  Prints offset to check

         printf ("offset = (%6.2f,%6.2f);\n",  x_offset, y_offset)
        
#  deleting file

         if (access (center_file)) delete (center_file, ver-, >& "dev$null")

 

#  Calculates the offset for images, relative to the first image and puts it
#    in the file 'center_file'


         lintran (files=Ixy_file, x2=x_offset, y2=y_offset, > center_file)


#doing photometry on each image...using file with corrections.

         printf ("phot...")

         delete ("star.mag,star.tab,star2.tab,star3.tab,star.col,star.dat,star2.dat", 
                 ver-, >& "dev$null")

         phot (image=img, coords=center_file, output="star.mag", 
                        calgori="none", 
			salgori="ofilter", annulus=10, dannulus=10, 
			apertur=2, datamin=INDEF, datamax=INDEF, 
                        verify-, interact-, verbose-)
# goto junk
         pconvert (textfile="star.mag", table="star.tab", fields="MAG,MERR")
         
	tinfo (table="star.tab",ttout-)
         nrows = tinfo.nrows
         nx = int(sqrt(nrows))
         ny = int((nrows - 0.5) / nx) + 1
        
	 printf ("nrows,nx,ny = %d,%d,%d;", nrows,nx,ny)
         
	npixels = nx * ny
        
	 extra_rows = npixels - nrows
        

	 if (extra_rows != 0) {
           tlcol (table="star.tab", nlist=4, > "star.col")
        
	   tcreate (table="star2.tab", cdfile="star.col", 
              datafile="star.junk",nrows=extra_rows)
        
	   tmerge (intable="star.tab,star2.tab", outtable="star3.tab",
              option="append")
        
  	 delete ("star.tab", ver-, >& "dev$null")
           rename (file="star3.tab", newname="star.tab")

         }

         tdump (table="star.tab",cdfile="",pfile="",datafile="star.dat",
            columns="", rows="-")

         sh kill_indef.sh

         delete ("star.tab", ver-, >& "dev$null")

         tcreate (table="star.tab", cdfile="star.col", 
              datafile="star2.dat",nrows=npixels)

         imdel (img//"s", >& "dev$null")

# Create an image from the table 'star.tab'..where each pixel represents a
#   star. 
         tabim (intable="star.tab", output=img//"s", colname="MAG", 
                ndim=2, n1=nx, n2=ny)

# naming each image...for use later on 	

	delete ("image"//i, ver-, >& "dev&null")

	img = "image"//i		

	printf("i = %d;", i) 

      } else {
         print ("LIGHTCURVE: reached premature EOF on offsets file")
         beep ; goto cleanup
      }
   }	 
}


#	delete ("j1", ver-, >&"dev$null")

#	imexpression (expr=min(a,b,c,d,e,f,g,h), out="j1", a="image")
# .......





cleanup:

   IMGLIST = ""
   XY_FILE = ""
   OFFSETS = ""

# Delete temporary files

   delete (imgfile, ver-, >& "dev$null")
   delete (center_file, ver-, >& "dev$null")


end






