procedure flat3 (input, out)
#  Task to take a series of flat field images and
# produce a 'super' flat from them. It is assumed
# that the flat field images have been processed by
# aptproc2 to remove the ADC bias level, and the
# non-linearity in the CCD sensitivity
#
#  D.Marlay 10/2/97


string	input	{prompt="files to be used in superflat"}
string	out	{prompt="output superflat file"}
string	*imglist

begin

  string	templist
  string	filelist
  string	image
  string	temp1, temp2
  real		mean
  real		stddev

  string	outfile
  string	infiles

  outfile = out
  imdelete (outfile, ver-)
  infiles = input

  unlearn sections
  unlearn imcombine
  unlearn imarith
  unlearn imstatistics
  unlearn imexpr
  unlearn imdelete
  unlearn hedit

  templist = mktemp ("templist")
  temp1 = mktemp ("temp1")
  temp2 = mktemp ("temp2")
  temp1 = "t1"
  temp2 = "t2"
  imdelete (temp1, ver-)
  imdelete (temp2, ver-)

#  Create a list of filenames from the input template.
  sections (infiles, option='fullname', > templist)
  imglist = templist
  filelist = ""
  while (fscan (imglist,image) != EOF) {
#    hedit (images=image, fields="BPM", delete+,add-, verify-)
    filelist = filelist//","//image
  }
#  printf ("Completed first stage")

#  Use imcombine to create the first superflat image.
#  imcombine (input=infiles , output=temp1, combine="median", reject="avsigclip", hsigma=1, scale="median")
  imcombine (input=infiles , output=temp2, combine="average",
     reject="avsigclip", scale="median", weight="median", mclip=yes, lsigma=3,
     hsigma=1.5)

  imstatistics (images=temp2, fields="mean", format-) | scan (mean)
  imarith (temp2, "/", mean, outfile)
  imdelete (temp2)
  bye

#  For flat field each image, then create a bit mask for the pixels containing
# stars to mask them out of the next pass of imcombine.
  imglist = templist
  while (fscan (imglist,image) != EOF) {
    i = strlen (image)
    if (substr (image,i-3,i) == ".imh") {
      image = substr (image,1,i-4)
    }
    imdelete (image//"mask", ver-)
    imdelete (image//"flat",ver-)
    imarith (image, "/", outfile, image//"flat")
    imstatistics (images=image//"flat", fields="mean,stddev", format-) | scan (mean,stddev)
    imexpr ("a > "//str(mean+2*stddev)//" ? 10 : 0", image//"mask", image//"flat")
    imdelete (image//".pl",ver-)
    imcopy (input=image//"mask", output=image//".pl")
    imdelete (image//"mask")
    imdelete (image//"flat")
    hedit (images=image, fields="BPM", value=image//".pl",add+, delete-,verify-)
  }
  
  imglist = templist
  infiles = ""
  while (fscan (imglist,image) != EOF) {
    i = strlen (image)
    if (substr (image,i-3,i) == ".imh") {
      image = substr (image,1,i-4)
    }
    imdelete (image//"t1",ver-)
    imdelete (image//"t2",ver-)
    imarith (image, "/", outfile, image//"t1")
    imsurfit (image//"t1", image//"t2", 2, 2, cross_terms = no)
    imstatistics (images=image//"t2", fields="mean",format-) | scan (mean)
    imdelete (image//"t1",ver-)
    imarith (image//"t2", "/" , mean, image//"t1")
    imdelete (image//"t2",ver-)
    imarith (image, "/", image//"t1", image//"t2")
    infiles = infiles//","//image//"t2"
    imdelete (image//"t1",ver-)
  }







#  Make a second pass with imcombine.
  imdelete (outfile)
  imcombine (input=infiles, output=temp1, combine="average", reject="avsigclip", masktype="badvalue", maskvalue=10, scale="median", weight="median", hsigma=1)
  imstatistics (images=temp1, fields="mean", format-) | scan (mean)
  imdelete (outfile, ver-)
  imarith (temp1, "/", mean, outfile)
  imdelete (temp1, ver-)
  delete (templist, ver-)

end

















