procedure plot_var (DataSet)

# Michael Ashley / UNSW / 28-Apr-1998 updated 3 June 1998 - JMR
#                         28-Oct-1998 super-duper version - MCBA
#
# Allows the interactive examination of light curves for the 
# RR Lyrae project.
#
# task plot_var=scripts$plot_var.cl

string DataSet = "" {prompt="Base name for data set"}

struct *imglist

begin
  string IDataSet
  string ThisRoutine = "PlotVar"
  string ImagesFile, TemplateImageFile, PhotFile, SigmaFile 
  string CoordsFile, CoordsTextFile, CoordsTabFile, CoordsTabDefnFile
  string OffsetsFile, TimeTextFile, TimeTabFile, TimeTabDefnFile
  string MagTextFile, MagTabFile, MagTabDefnFile
  string StarTextFile, StarTabFile
  string tmpimg = ""
  string imgfile = ""
  string vec, WhatToPlot
  real   timei
  int    NumberOfImages
  struct command, img
  
  real   x, y, xcenter, ycenter, vmin, vmax
  int    wcs, ix, iy, n1, n2, nrows, id
  string key

#  string UparmDir

# Unlearn all the IRAF routines that we use.

#  UparmDir  = mktemp ("tmp$iraf")
#  printf ("dir = "//UparmDir//" \n")
#  mkdir UparmDir
#  set uparm = UparmDir

  unlearn pdump
  unlearn tcreate
  unlearn tabpar
  unlearn tinfo
  unlearn tdump
  unlearn delete
  unlearn sections
  unlearn imgets
  unlearn display
  unlearn tvmark
  unlearn imdelete
  unlearn sgraph

  IDataSet = DataSet

# Construct the filenames of files that we will use/create.

  ImagesFile        = IDataSet//".images"
  TemplateImageFile = IDataSet//"Template"
  PhotFile          = IDataSet//"Phot"
  SigmaFile         = IDataSet//"Sigma"
  CoordsFile        = IDataSet//"Coords.coo"
  CoordsTextFile    = IDataSet//"Coords.txt"
  CoordsTabFile     = IDataSet//"Coords.tab"
  CoordsTabDefnFile = IDataSet//"Coords.cd"
  OffsetsFile       = IDataSet//"Offsets"
  TimeTextFile      = IDataSet//"Times.txt"
  TimeTabFile       = IDataSet//"Times.tab"
  TimeTabDefnFile   = IDataSet//"Times.cd"
  MagTextFile       = IDataSet//"Mag.txt"
  MagTabFile        = IDataSet//"Mag.tab"
  MagTabDefnFile    = IDataSet//"Mag.cd"

# Form temporary filenames.

  imgfile = mktemp("tmp$iraf")
  tmpimg  = mktemp("tmp$iraf")

# Check the existence of the files that we need.

  if (!access(ImagesFile)) {
    print (ThisRoutine//": Image list file ", ImagesFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

  if (!access(PhotFile//".imh") && !access(PhotFile//".fits")) {
    print (ThisRoutine//": Photometry cube file ", PhotFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

  if (!access(TemplateImageFile//".imh") && !access(TemplateImageFile//".fits")) {
    print (ThisRoutine//": Template image file ", TemplateImageFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

  if (!access(SigmaFile//".imh") && !access(SigmaFile//".fits")) {
    print (ThisRoutine//": Sigma cube file ", SigmaFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

  if (!access(CoordsFile)) {
    print (ThisRoutine//": Coordinate file ", CoordsFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

  if (!access(OffsetsFile)) {
    print (ThisRoutine//": Image offsets file ", OffsetsFile,
         " does not exist - processing aborted.")
    beep; goto cleanup
  }

# ---------------------------------------------------------------------------
# Create a table (CoordsTabFile) containing the X,Y coordinates of each star.

# Delete the files we need to create.

  delete (CoordsTextFile,    ver-, >& "dev$null")
  delete (CoordsTabFile,     ver-, >& "dev$null")
  delete (CoordsTabDefnFile, ver-, >& "dev$null")

# Extract the (X,Y) center information from the CoordsFile.

  pdump (infiles=CoordsFile, fields="XCENTER,YCENTER", expr="yes",
          > CoordsTextFile)

# Construct a table definition file.

  print ("XCENTER r\nYCENTER r\n", > CoordsTabDefnFile)

# Create the table of (X,Y) information.

  tcreate (table    = CoordsTabFile, 
           cdfile   = CoordsTabDefnFile, 
           datafile = CoordsTextFile) 

# ---------------------------------------------------------------------------
# Create files (TimeTextFile and TimeTabFile) containing the 
# timing information from the images.

# Delete the files we need to create.

  delete (TimeTextFile,    ver-, >& "dev$null")
  delete (TimeTabFile,     ver-, >& "dev$null")
  delete (TimeTabDefnFile, ver-, >& "dev$null")

# Extract the full names of the images.

  sections ("@"//ImagesFile, option="fullname", > imgfile)
  NumberOfImages = sections.nimages
  if (NumberOfImages == 0) {
    print (ThisRoutine//": No images in input list ", ImagesFile,
           " - processing aborted.")
    beep; goto cleanup
  }

# Extract the timing information into a text file.

  imglist = imgfile

  while (fscan(imglist, img) != EOF) { 
    if (!access(img) && !access(img//".fits") ) {
      print (ThisRoutine//": Image ", img," does not exist")
         beep; goto cleanup
    } else {
      if (stridx('.',img) != 0) {
        img = substr(img,1,stridx('.',img)-1)
      }
      printf ("\nProcessing %s ...", img)
          
      imgets (image=img, param="JD")
      timei = real(imgets.value) - 2450000.00000
      printf ("%.5f\n", timei, >> TimeTextFile)
    }
  }

# Now create a table with the timing information. Begin by creating a
# table definition file.

  print ("TIME   r\n", > TimeTabDefnFile)

  tcreate (table    = TimeTabFile, 
           cdfile   = TimeTabDefnFile,
           datafile = TimeTextFile) 

# ---------------------------------------------------------------------------

# Construct a table definition file.

  delete (MagTabDefnFile, ver-, >& "dev$null")
  print ("FRAME r\nMAG r\n", > MagTabDefnFile)

  display (image=SigmaFile, frame=1, zscale+, contrast=0.1,
           zrange-)

  imgets (image=PhotFile, param="i_naxis1")
  n1 = int (imgets.value)
  imgets (image=PhotFile, param="i_naxis2")
  n2 = int (imgets.value)
  tinfo (table=CoordsTabFile, ttout-)
  nrows = tinfo.nrows

# If you change the following, update it further down as well.

  printf ("  i - image: display the star in the template image\n"//
          "  m - matrix: display the star in all the images\n"//
          "  p - plot the photometry data versus image number\n"//
          "  t - plot the photometry data versus time\n"//
          "  a - analysis: perform PDM analysis\n"//
          "  q - quit\n")

  while (fscan (imcur, x, y, wcs, command) != EOF) {
    key = substr (command, 1, 1)
    if (key == "q")
      break
    
    ix = int(0.5 + x)
    iy = int(0.5 + y)
    if (ix < 1 || ix > n1 || iy < 1 || iy > n2) {
      beep; printf ("outside the image\n"); next
    }
    id = ix + (iy - 1) * n1
    if (id < 1 || id > nrows) {
      beep; printf ("outside the range of stars\n"); next
    }
    tabpar (table=CoordsTabFile, column="XCENTER", row=id)
    xcenter = real (tabpar.value)
    tabpar (table=CoordsTabFile, column="YCENTER", row=id)
    ycenter = real (tabpar.value)
  
    printf ("star ID%5d @ (%8.2f,%8.2f)\n", 
                  id, xcenter, ycenter)

    if (key == "i") {
      generate_matrix (images=TemplateImageFile, output=tmpimg,
                       xcen=xcenter, ycen=ycenter, boxsize=20, 
                       shifts=OffsetsFile, verbose-)
      display (image=tmpimg, frame=2,
               zrange+, xmag=10, ymag=10)
      printf ("hit any key to continue...\n")
      if (fscan (imcur, x, y, wcs, command) != EOF)  {
      }
      tvmark (frame=1, coords="", interact-)

    } else if (key == "m") {
      printf ("calculating matrix, please wait ...")
      generate_matrix (images="@"//ImagesFile, output=tmpimg,
                       xcen=xcenter, ycen=ycenter, boxsize=20, 
                       shifts=OffsetsFile, verbose-)
      display (image=tmpimg, frame=2, zrange+)
      printf ("hit any key to continue...\n")
      if (fscan (imcur, x, y, wcs, command) != EOF)  {
      }
      tvmark (frame=1, coords="", interact-)

    } else if ((key == "p") || (key == "t") || (key == "a")) {

# Extract the star photometry data.

      vec = PhotFile//"["//ix//":"//ix//","//iy//":"//iy//",*]"

      delete (MagTabFile,  ver-, >& "dev$null")
      delete (MagTextFile, ver-, >& "dev$null")

      listpixels (images=vec, > MagTextFile)

# Create the table of (TIME,MAG) information.

      tcreate (table    = MagTabFile, 
               cdfile   = MagTabDefnFile, 
               datafile = MagTextFile)

      StarTabFile  = "star_"//id//".tab"
      StarTextFile = "star_"//id//".txt"

      delete (StarTabFile,  ver-, >& "dev$null")
      delete (StarTextFile, ver-, >& "dev$null")

      tmerge  (intable  = TimeTabFile//","//MagTabFile,
               outtable = StarTabFile,
               option   = "merge")

      tdump   (table    = StarTabFile,
               datafile = StarTextFile,
               columns  = "TIME,MAG",
               >& "dev$null")

# Find the range of the data, ignoring the 0.0 values which result from
# photometry errors.

      imstatistics (images=vec, lower=1.0, fields="min,max", format-) | 
            scan (vmin, vmax)

      if (key == "t") {
        WhatToPlot = StarTabFile//" TIME MAG"
      } else {
        WhatToPlot = StarTabFile//" FRAME MAG"
      }

# Ensure that the range plotted is at least 0.2 magnitudes; this helps
# determine whether the star variation is significant or not.

      if (vmax - vmin > 0.2) {
        axispar.wb = vmin
        axispar.wt = vmax
        sgraph (WhatToPlot, point+, marker="circle", szmarker=7)
      } else {
        axispar.wb = -0.1 + (vmin+vmax)/2
        axispar.wt =  0.1 + (vmin+vmax)/2
        sgraph (WhatToPlot, point+, marker="circle", szmarker=3)
      }

      if (key == "a") {
        pdm (infiles = StarTextFile)
      }

    } else {
      printf ("  i - image: display the star in the template image\n"//
              "  m - matrix: display the star in all the images\n"//
              "  p - plot the photometry data versus image number\n"//
              "  t - plot the photometry data versus time\n"//
              "  a - analysis: perform PDM analysis\n"//
              "  q - quit\n")
    }
  }
  goto cleanup

cleanup:
  if (access(imgfile)) {
    delete (imgfile, ver-, >& "dev$null")
  }
  imdelete (tmpimg, ver-, >& "dev$null")
#  delete (UparmDir//".par", ver-, >& "dev$null")
#  delete (UparmDir, ver-, >& "dev$null")
   beep
end
